% this script (dirsig_focus.m) is loosely based upon the spotlight SAR
% matlab code originally written by Leroy Gorham and presented at SPIE
%   LeRoy A. Gorham ; Linda J. Moore; SAR image formation toolbox for
%      MATLAB. Proc. SPIE 7699, Algorithms for Synthetic Aperture Radar 
%      Imagery XVII, 769906 (April18, 2010).

clear all
clc
close all

cph_filename = '../demo.img';
ppd_filename = '../stripmap_200mps.ppd';
platform_filename = '../demo.platform';

nsamples    = 3201;   % number of range samples in phase history
npulses     = 400;   % number of pulses in aperture
startpulse  = 0;

imagesize   = 400;   % spatial sample size in focused image
gsd         = 0.25;  % ground sample distance in meters
fprintf('Image to be formed with a spatial extent of %f meters on a side\n', imagesize*gsd);

out = dir( cph_filename );
pcount = out.bytes/nsamples/16.0;
fprintf('total of %f pulses in phase history file.\n',pcount);
ph = readcph(cph_filename, nsamples, npulses, startpulse );
[ppd_x, ppd_y, ppd_z, ppd_t] = readppd(ppd_filename);
[chirprate, tau, fc, t0, t1, dt, prf] = readplatform(platform_filename);
im_final = zeros(imagesize,imagesize);
data.phdata = ph;

figure(1);
phMax = max(max(abs(ph)));
imshow(abs(ph)',[0, phMax]);
colormap('default');

% Position of the sensor for a circular SAR collection
t = (0:npulses-1) / prf;
data.AntX = zeros(1, npulses);
data.AntY = zeros(1, npulses);
data.AntZ = zeros(1, npulses);
theta     = zeros(1, npulses);

% linearly interpolate position information from PPD, just in case
% it is not evenly spaced and/or on the same time posts as the actual
% phase history is
for ii=1:npulses
    data.AntX(ii) =  interp1(ppd_t,ppd_x,t(ii),'linear');
    data.AntY(ii) =  interp1(ppd_t,ppd_y,t(ii),'linear');
    data.AntZ(ii) =  interp1(ppd_t,ppd_z,t(ii),'linear');
    theta(ii) = atan2( data.AntY(ii), data.AntX(ii) ) * 180.0 / pi;
end

% center and total frequency range of collection
data.fc     = fc;
data.BW     = chirprate * tau;
data.deltaF = data.BW / nsamples;
data.freq = (data.fc - data.BW/2.0):data.deltaF:(data.fc + data.BW/2.0);

% Size of FFT to form range profile
data.Nfft = 2048;   %2^12

% X,Y and Z position of each pixel (m)
halfimageextent = 0.5 * gsd * imagesize;
[data.x_mat,data.y_mat] = meshgrid(-halfimageextent:gsd:halfimageextent-gsd, ...
                                   -halfimageextent:gsd:halfimageextent-gsd);
data.xmat=data.x_mat;
data.ymat=data.y_mat;
data.zmat = zeros(imagesize,imagesize);
R0 = zeros(300) + 15000.0 * sqrt(2.0);

% Define speed of light (m/s)
c = 299792458;

% Determine the size of the phase history data
data.K = size(data.phdata,1); % The number of frequency bins per pulse
data.Np = size(data.phdata,2); % The number of pulses

data.alpha = chirprate;

% Determine the azimuth angles of the image pulses (radians)
data.AntAz = unwrap(atan2(data.AntY,data.AntX));

% Determine the average azimuth angle step size (radians)
data.deltaAz = abs(mean(diff(data.AntAz)));

% Determine the total azimuth angle of the aperture (radians)
data.totalAz = max(data.AntAz) - min(data.AntAz);

% Determine the maximum scene size of the image (m)
data.maxWr = c/(2*data.deltaF);
data.maxWx = c/(2*data.deltaAz*mean(data.freq));

% Determine the resolution of the image (m)
data.dr = c/(2*data.deltaF*data.K);
data.dx = c/(2*data.totalAz*mean(data.freq));

% Display maximum scene size and resolution
fprintf('Maximum Scene Size: %.1f m range, %.1f m cross-range, Azimuth angle: %d\n',data.maxWr,data.maxWx,j);
fprintf('Resolution: %.3fm range, %.3f m cross-range\n',data.dr,data.dx);

% Calculate the range to every bin in the range profile (m)
data.r_vec = linspace(-data.Nfft/2,data.Nfft/2-1,data.Nfft)*data.maxWr/data.Nfft;

% Initialize the image with all zero values
%data.im_final = zeros(size(data.x_mat));

% Set up a vector to keep execution times for each pulse (sec)
t = zeros(1,data.Np);

% Calculate the Taylor windows in the range and azimuth directions
tw_rng = taylorwin( data.K );
tw_az  = taylorwin( data.Np );

% Loop through every pulse in the synthetic aperture and backproject into
% image grid
for ii = 1:data.Np
   
    tfast = (zeros(1, data.K) + 2.0 * R0(ii) / c) + (-data.K/2:1:data.K/2-1)*dt;
    tofs = -1.0 * ( tfast - 2.0 * R0(ii) / c );
    tofsq = tofs .* tofs;
    mxphase = 2.0 * pi * data.fc * tofs + data.alpha * tofsq - 2.0 * pi * data.fc;
    mxphase = mod( mxphase, 2.0 * pi );
    
    % Form the range profile with zero padding added by mixing with
    % reference signal and then performing a discrete FFT
    mxsg = complex( cos(-mxphase), sin(-mxphase));
    mixeddata = zeros( 1, data.Nfft );
    for jj = 1:data.K
        mixeddata(jj) = data.phdata(jj,ii) .* mxsg(jj) .* tw_rng(jj) .* tw_az(ii);
    end
    rc = fftshift(ifft(mixeddata,data.Nfft));

    % Calculate differential range for each pixel in the image (m) relative
    % to the platform for this pulses
    dR = sqrt((data.AntX(ii)-data.xmat).^2 + ...
             (data.AntY(ii)- data.ymat).^2 + ...
             (data.AntZ(ii)- data.zmat).^2) - R0(ii);

    % Calculate phase correction for image
    phCorr = exp( 1i*4*pi*data.freq(1)/c*dR );
    
    % Determine which pixels fall within the range swath
    I = find(and(dR > min(data.r_vec), dR < max(data.r_vec)));

    % Update the image using linear interpolation
    if (sum(isnan(rc)) == 0) 
        im_final(I) = im_final(I) + interp1(data.r_vec,rc,dR(I),'linear') .* phCorr(I);
        if mod(ii,10) == 0
            figure(2);
            mmx = max(max(abs(im_final)));
            imshow(abs(im_final)',[0, mmx]);
            colormap('default');
            pngName = strcat('ph_', num2str(ii), '.png');
            pngData = abs(im_final)/mmx*255;
            imwrite(pngData',jet(255),pngName,'png');
        end
    else
        fprintf('NaN found in range profile (ii=%d)\n',ii);
    end

end

figure(2);
mx = max(max((abs(im_final))));
imshow(log10(abs(im_final)/mx)',[-2 0]);
colormap('default');
